@@ -6,7 +6,7 @@ Base CAPTCHA stuff
 import enum
 import importlib
 from abc import ABC
-from dataclasses import dataclass, MISSING
+from dataclasses import dataclass, fields, MISSING
 from typing import Dict
 
 
@@ -19,9 +19,10 @@ class CaptchaType(enum.Enum):
     TEXT = "TextCaptcha"
     FUNCAPTCHA = "FunCaptcha"
     GEETEST = "GeeTest"
+    GEETESTV4 = "GeeTestV4"
     HCAPTCHA = "HCaptcha"
     KEYCAPTCHA = "KeyCaptcha"
-    CAPY = "Capy"
+    CAPY = "CapyPuzzle"
     TIKTOK = "TikTokCaptcha"
 
 
@@ -55,8 +56,8 @@ class BaseCaptcha(ABC):
         if not kwargs:
             # get all optional params
             kwargs = {
-                k: (k, None) for k, v in self.__dataclass_fields__.items()
-                if v.default is not MISSING
+                field.name: (field.name, None) for field in fields(self)
+                if field.default is not MISSING
             }
 
         for opt_field in kwargs:
@@ -77,13 +78,16 @@ class BaseCaptchaSolution(ABC):
     def get_type(cls) -> CaptchaType:
         """ Returns CaptchaType """
 
-        return CaptchaType(cls.__name__.split("Solution")[0])
+        return CaptchaType(cls.__name__.split("Solution", maxsplit=1)[0])
 
     @classmethod
     def get_captcha_class(cls) -> BaseCaptcha:
         """ Returns appropriate solution class """
 
-        return getattr(importlib.import_module(cls.__module__), cls.__name__.split("Solution")[0])
+        return getattr(
+            importlib.import_module(cls.__module__),
+            cls.__name__.split("Solution", maxsplit=1)[0]
+        )
 
     def __str__(self):
-        return '\n'.join([getattr(self, field) for field in self.__dataclass_fields__])
+        return '\n'.join(str(getattr(self, field.name)) for field in fields(self))
