@@ -5,7 +5,9 @@ Certain services related stuff
 
 import enum
 # pylint: disable=import-self
-from . import anti_captcha, azcaptcha, cptch_net, rucaptcha, twocaptcha
+from . import (
+    anti_captcha, azcaptcha, captcha_guru, cptch_net, rucaptcha, twocaptcha
+)
 
 
 class CaptchaSolvingService(enum.Enum):
@@ -13,6 +15,7 @@ class CaptchaSolvingService(enum.Enum):
 
     ANTI_CAPTCHA = "anti-captcha.com"
     AZCAPTCHA = "azcaptcha.com"
+    CAPTCHA_GURU = "captcha.guru"
     CPTCH_NET = "cptch.net"
     RUCAPTCHA = "rucaptcha.com"
     TWOCAPTCHA = "2captcha.com"
@@ -22,6 +25,7 @@ class CaptchaSolvingService(enum.Enum):
 SOLVING_SERVICE = {
     CaptchaSolvingService.ANTI_CAPTCHA: anti_captcha,
     CaptchaSolvingService.AZCAPTCHA: azcaptcha,
+    CaptchaSolvingService.CAPTCHA_GURU: captcha_guru,
     CaptchaSolvingService.CPTCH_NET: cptch_net,
     CaptchaSolvingService.RUCAPTCHA: rucaptcha,
     CaptchaSolvingService.TWOCAPTCHA: twocaptcha
