@@ -3,8 +3,6 @@
 2captcha.com service
 """
 
-import json
-
 from .base import HTTPService
 from .._transport.http_transport import HTTPRequestJSON  # type: ignore
 from .. import exceptions
@@ -21,8 +19,10 @@ __all__ = [
     'FunCaptchaTaskRequest', 'FunCaptchaSolutionRequest',
     'KeyCaptchaTaskRequest', 'KeyCaptchaSolutionRequest',
     'GeeTestTaskRequest', 'GeeTestSolutionRequest',
+    'GeeTestV4TaskRequest', 'GeeTestV4SolutionRequest',
     'HCaptchaTaskRequest', 'HCaptchaSolutionRequest',
-    'CapyTaskRequest', 'CapySolutionRequest'
+    'CapyPuzzleTaskRequest', 'CapyPuzzleSolutionRequest',
+    'TikTokCaptchaTaskRequest', 'TikTokCaptchaSolutionRequest'
 ]
 
 
@@ -34,16 +34,18 @@ class Service(HTTPService):
     def _post_init(self):
         """ Init settings """
 
-        for captcha_type in self._settings:
-            self._settings[captcha_type].polling_interval = 5
-            self._settings[captcha_type].solution_timeout = 180
+        for captcha_type in self.settings:
+            self.settings[captcha_type].polling_delay = 5
+            self.settings[captcha_type].polling_interval = 5
+            self.settings[captcha_type].solution_timeout = 180
+
             if captcha_type in (CaptchaType.RECAPTCHAV2, CaptchaType.HCAPTCHA):
-                self._settings[captcha_type].polling_delay = 20
-                self._settings[captcha_type].solution_timeout = 300
+                self.settings[captcha_type].polling_delay = 20
+                self.settings[captcha_type].solution_timeout = 300
             elif captcha_type in (CaptchaType.RECAPTCHAV3,):
-                self._settings[captcha_type].polling_delay = 15
-            else:
-                self._settings[captcha_type].polling_delay = 5
+                self.settings[captcha_type].polling_delay = 15
+            elif captcha_type in (CaptchaType.TIKTOK,):
+                self.settings[captcha_type].polling_interval = 1
 
 
 class Request(HTTPRequestJSON):
@@ -60,36 +62,36 @@ class Request(HTTPRequestJSON):
         ###############
         # handle errors
         ###############
-        error = response_data["request"]
+        error_code = response_data["request"]
         error_text = response_data.get("error_text", "")
-        error_msg = "{}: {}".format(error, error_text)
+        error_msg = f"{error_code}: {error_text}"
 
-        if error == 'CAPCHA_NOT_READY':  # pylint: disable=no-else-raise
+        if error_code == 'CAPCHA_NOT_READY':  # pylint: disable=no-else-raise
             raise exceptions.SolutionNotReadyYet()
-        elif error in ('ERROR_WRONG_USER_KEY', 'ERROR_KEY_DOES_NOT_EXIST', 'ERROR_IP_NOT_ALLOWED',
-                       'IP_BANNED'):
+        elif error_code in ('ERROR_WRONG_USER_KEY', 'ERROR_KEY_DOES_NOT_EXIST',
+                            'ERROR_IP_NOT_ALLOWED', 'IP_BANNED'):
             raise exceptions.AccessDeniedError(error_msg)
-        elif error in ('ERROR_ZERO_BALANCE',):
+        elif error_code in ('ERROR_ZERO_BALANCE',):
             raise exceptions.LowBalanceError(error_msg)
-        elif error in ('ERROR_NO_SLOT_AVAILABLE',):
+        elif error_code in ('ERROR_NO_SLOT_AVAILABLE',):
             # If server returns ERROR_NO_SLOT_AVAILABLE make a 5 seconds timeout before sending
             # next request.
             # time.sleep(5)
             raise exceptions.ServiceTooBusy(error_msg)
-        elif error in ('MAX_USER_TURN',) or error.startswith('ERROR:'):
+        elif error_code in ('MAX_USER_TURN',) or error_code.startswith('ERROR:'):
             raise exceptions.TooManyRequestsError(error_msg)
-        elif error in ('ERROR_WRONG_ID_FORMAT', 'ERROR_WRONG_CAPTCHA_ID'):
+        elif error_code in ('ERROR_WRONG_ID_FORMAT', 'ERROR_WRONG_CAPTCHA_ID'):
             raise exceptions.MalformedRequestError(error_msg)
-        elif error in ('ERROR_ZERO_CAPTCHA_FILESIZE', 'ERROR_TOO_BIG_CAPTCHA_FILESIZE',
-                       'ERROR_WRONG_FILE_EXTENSION', 'ERROR_IMAGE_TYPE_NOT_SUPPORTED',
-                       'ERROR_UPLOAD', 'ERROR_PAGEURL', 'ERROR_BAD_TOKEN_OR_PAGEURL',
-                       'ERROR_GOOGLEKEY', 'ERROR_BAD_PARAMETERS', 'ERROR_TOKEN_EXPIRED',
-                       'ERROR_EMPTY_ACTION'):
+        elif error_code in ('ERROR_ZERO_CAPTCHA_FILESIZE', 'ERROR_TOO_BIG_CAPTCHA_FILESIZE',
+                            'ERROR_WRONG_FILE_EXTENSION', 'ERROR_IMAGE_TYPE_NOT_SUPPORTED',
+                            'ERROR_UPLOAD', 'ERROR_PAGEURL', 'ERROR_BAD_TOKEN_OR_PAGEURL',
+                            'ERROR_GOOGLEKEY', 'ERROR_BAD_PARAMETERS', 'ERROR_TOKEN_EXPIRED',
+                            'ERROR_EMPTY_ACTION'):
             raise exceptions.BadInputDataError(error_msg)
-        elif error in ('ERROR_CAPTCHAIMAGE_BLOCKED', 'ERROR_CAPTCHA_UNSOLVABLE',
-                       'ERROR_BAD_DUPLICATES'):
+        elif error_code in ('ERROR_CAPTCHAIMAGE_BLOCKED', 'ERROR_CAPTCHA_UNSOLVABLE',
+                            'ERROR_BAD_DUPLICATES'):
             raise exceptions.UnableToSolveError(error_msg)
-        elif error in ('ERROR_BAD_PROXY', 'ERROR_PROXY_CONNECTION_FAILED'):
+        elif error_code in ('ERROR_BAD_PROXY', 'ERROR_PROXY_CONNECTION_FAILED'):
             raise exceptions.ProxyError(error_msg)
 
         raise exceptions.ServiceError(error_msg)
@@ -98,10 +100,10 @@ class Request(HTTPRequestJSON):
 class InRequest(Request):
     """ Request class for requests to /in.php """
 
-    def prepare(self) -> dict:
+    def prepare(self, **kwargs) -> dict:
         """ Prepare request """
 
-        request = super().prepare()
+        request = super().prepare(**kwargs)
         request.update(
             dict(
                 method="POST",
@@ -119,10 +121,10 @@ class InRequest(Request):
 class ResRequest(Request):
     """ Request class for requests to /res.php """
 
-    def prepare(self) -> dict:
+    def prepare(self, **kwargs) -> dict:
         """ Prepare request """
 
-        request = super().prepare()
+        request = super().prepare(**kwargs)
         request.update(
             dict(
                 method="GET",
@@ -171,7 +173,7 @@ class ReportGoodRequest(ResRequest):
     def prepare(self, solved_captcha) -> dict:  # type: ignore
         """ Prepare request """
 
-        request = super().prepare()
+        request = super().prepare(solved_captcha=solved_captcha)
         request["params"].update(
             dict(
                 action="reportgood",
@@ -188,7 +190,7 @@ class ReportBadRequest(ResRequest):
     def prepare(self, solved_captcha) -> dict:  # type: ignore
         """ Prepare request """
 
-        request = super().prepare()
+        request = super().prepare(solved_captcha=solved_captcha)
         request["params"].update(
             dict(
                 action="reportbad",
@@ -202,8 +204,15 @@ class TaskRequest(InRequest):
     """ Common Task Request class """
 
     # pylint: disable=arguments-differ,unused-argument
-    def prepare(self, captcha=None, proxy=None, user_agent=None, cookies=None):
-        request = super().prepare()
+    def prepare(self, captcha, proxy, user_agent, cookies):
+        """ Prepare a request """
+
+        request = super().prepare(
+            captcha=captcha,
+            proxy=proxy,
+            user_agent=user_agent,
+            cookies=cookies
+        )
 
         if proxy:
             request['data'].update(
@@ -239,10 +248,7 @@ class SolutionRequest(ResRequest):
     def prepare(self, task) -> dict:  # type: ignore
         """ Prepare request """
 
-        # save task
-        self._task = task
-
-        request = super().prepare()
+        request = super().prepare(task=task)
         request["params"].update(
             dict(action="get2", id=task.task_id)
         )
@@ -254,26 +260,40 @@ class SolutionRequest(ResRequest):
         response_data = super().parse_response(response)
 
         solution_data = response_data.pop("request")
-        solution_class = self._task.captcha.get_solution_class()
-        if self._task.captcha.get_type() in (CaptchaType.GEETEST, CaptchaType.CAPY):
-            try:
-                solution_dict = json.loads(solution_data)
-            except json.JSONDecodeError as exc:
-                raise exceptions.ServiceError('Got unrecognized data from the service!') from exc
-
-            solution = solution_class(**solution_dict)
-        elif self._task.captcha.get_type() in (CaptchaType.TIKTOK,):
+        solution_class = self.source_data['task'].captcha.get_solution_class()
+        captcha_type = self.source_data['task'].captcha.get_type()
+        if captcha_type == CaptchaType.GEETEST:
+            solution = solution_class(
+                challenge=solution_data['geetest_challenge'],
+                validate=solution_data['geetest_validate'],
+                seccode=solution_data['geetest_seccode']
+            )
+        elif captcha_type == CaptchaType.GEETESTV4:
+            solution = solution_class(
+                captcha_id=solution_data['captcha_id'],
+                lot_number=solution_data['lot_number'],
+                pass_token=solution_data['pass_token'],
+                gen_time=solution_data['gen_time'],
+                captcha_output=solution_data['captcha_output'],
+            )
+        elif captcha_type == CaptchaType.CAPY:
+            solution = solution_class(**solution_data)
+        elif captcha_type in (CaptchaType.TIKTOK,):
             solution = solution_class(
                 dict(
-                    [key_value.split(':') for key_value in solution_data.split(';')]
+                    [key_value.split(':', maxsplit=1) for key_value in solution_data.split(';')]
                 )
             )
         else:
             solution = solution_class(solution_data)
 
+        price = None
+        if 'price' in response_data:
+            price = response_data.pop("price")
+
         return dict(
             solution=solution,
-            cost=response_data.pop("price"),
+            cost=price,
             extra=response_data
         )
 
@@ -285,10 +305,20 @@ class ImageCaptchaTaskRequest(TaskRequest):
     def prepare(self, captcha, proxy, user_agent, cookies) -> dict:  # type: ignore
         """ Prepare request """
 
-        request = super().prepare()
+        request = super().prepare(
+            captcha=captcha,
+            proxy=proxy,
+            user_agent=user_agent,
+            cookies=cookies
+        )
 
         # add required params
-        request['data'].update(dict(method="base64", body=captcha.get_image_base64()))
+        request['data'].update(
+            dict(
+                method="base64",
+                body=captcha.get_image_base64().decode('ascii')
+            )
+        )
 
         # add optional params
         request['data'].update(
@@ -322,6 +352,7 @@ class RecaptchaV2TaskRequest(TaskRequest):
         """ Prepare request """
 
         request = super().prepare(
+            captcha=captcha,
             proxy=proxy,
             user_agent=user_agent,
             cookies=cookies
@@ -339,6 +370,7 @@ class RecaptchaV2TaskRequest(TaskRequest):
         request['data'].update(
             captcha.get_optional_data(
                 data_s=('data-s', None),
+                api_domain=('domain', None)
             )
         )
 
@@ -360,7 +392,12 @@ class RecaptchaV3TaskRequest(TaskRequest):
     def prepare(self, captcha, proxy, user_agent, cookies) -> dict:  # type: ignore
         """ Prepare request """
 
-        request = super().prepare(proxy=proxy)
+        request = super().prepare(
+            captcha=captcha,
+            proxy=proxy,
+            user_agent=user_agent,
+            cookies=cookies
+        )
         request['data'].update(
             dict(
                 method="userrecaptcha",
@@ -375,6 +412,7 @@ class RecaptchaV3TaskRequest(TaskRequest):
             captcha.get_optional_data(
                 action=('action', None),
                 min_score=('min_score', None),
+                api_domain=('domain', None)
             )
         )
 
@@ -396,7 +434,13 @@ class TextCaptchaTaskRequest(TaskRequest):
     def prepare(self, captcha, proxy, user_agent, cookies) -> dict:  # type: ignore
         """ Prepare request """
 
-        request = super().prepare()
+        request = super().prepare(
+            captcha=captcha,
+            proxy=proxy,
+            user_agent=user_agent,
+            cookies=cookies
+        )
+
         request['data'].update(
             dict(
                 textcaptcha=captcha.text
@@ -427,7 +471,13 @@ class FunCaptchaTaskRequest(TaskRequest):
     def prepare(self, captcha, proxy, user_agent, cookies) -> dict:  # type: ignore
         """ Prepare request """
 
-        request = super().prepare(proxy=proxy, user_agent=user_agent)
+        request = super().prepare(
+            captcha=captcha,
+            proxy=proxy,
+            user_agent=user_agent,
+            cookies=cookies
+        )
+
         request['data'].update(
             dict(
                 method="funcaptcha",
@@ -440,7 +490,8 @@ class FunCaptchaTaskRequest(TaskRequest):
         request['data'].update(
             captcha.get_optional_data(
                 service_url=('surl', None),
-                no_js=('nojs', None)
+                no_js=('nojs', None),
+                blob=('data[blob]', None),
             )
         )
 
@@ -458,7 +509,13 @@ class KeyCaptchaTaskRequest(TaskRequest):
     def prepare(self, captcha, proxy, user_agent, cookies) -> dict:  # type: ignore
         """ Prepare request """
 
-        request = super().prepare()
+        request = super().prepare(
+            captcha=captcha,
+            proxy=proxy,
+            user_agent=user_agent,
+            cookies=cookies
+        )
+
         request['data'].update(
             dict(
                 method="keycaptcha",
@@ -478,13 +535,19 @@ class KeyCaptchaSolutionRequest(SolutionRequest):
 
 
 class GeeTestTaskRequest(TaskRequest):
-    """ KeyCaptcha task request """
+    """ GeeTest task request """
 
     # pylint: disable=arguments-differ,signature-differs
     def prepare(self, captcha, proxy, user_agent, cookies) -> dict:  # type: ignore
         """ Prepare request """
 
-        request = super().prepare()
+        request = super().prepare(
+            captcha=captcha,
+            proxy=proxy,
+            user_agent=user_agent,
+            cookies=cookies
+        )
+
         request['data'].update(
             dict(
                 method="geetest",
@@ -505,7 +568,7 @@ class GeeTestTaskRequest(TaskRequest):
 
 
 class GeeTestSolutionRequest(SolutionRequest):
-    """ KeyCaptcha solution request """
+    """ GeeTest solution request """
 
 
 class HCaptchaTaskRequest(TaskRequest):
@@ -515,12 +578,26 @@ class HCaptchaTaskRequest(TaskRequest):
     def prepare(self, captcha, proxy, user_agent, cookies) -> dict:  # type: ignore
         """ Prepare request """
 
-        request = super().prepare(proxy=proxy)
+        request = super().prepare(
+            captcha=captcha,
+            proxy=proxy,
+            user_agent=user_agent,
+            cookies=cookies
+        )
+
         request['data'].update(
             dict(
                 method="hcaptcha",
                 sitekey=captcha.site_key,
-                pageurl=captcha.page_url
+                pageurl=captcha.page_url,
+                invisible=int(captcha.is_invisible)
+            )
+        )
+
+        # add optional params
+        request['data'].update(
+            captcha.get_optional_data(
+                api_domain=('domain', None)
             )
         )
 
@@ -531,14 +608,20 @@ class HCaptchaSolutionRequest(SolutionRequest):
     """ HCaptcha solution request """
 
 
-class CapyTaskRequest(TaskRequest):
-    """ Capy task request """
+class CapyPuzzleTaskRequest(TaskRequest):
+    """ CapyPuzzle task request """
 
     # pylint: disable=arguments-differ,signature-differs
     def prepare(self, captcha, proxy, user_agent, cookies) -> dict:  # type: ignore
         """ Prepare request """
 
-        request = super().prepare(proxy=proxy)
+        request = super().prepare(
+            captcha=captcha,
+            proxy=proxy,
+            user_agent=user_agent,
+            cookies=cookies
+        )
+
         request['data'].update(
             dict(
                 method="capy",
@@ -557,8 +640,8 @@ class CapyTaskRequest(TaskRequest):
         return request
 
 
-class CapySolutionRequest(SolutionRequest):
-    """ Capy solution request """
+class CapyPuzzleSolutionRequest(SolutionRequest):
+    """ CapyPuzzle solution request """
 
 
 class TikTokCaptchaTaskRequest(TaskRequest):
@@ -568,11 +651,38 @@ class TikTokCaptchaTaskRequest(TaskRequest):
     def prepare(self, captcha, proxy, user_agent, cookies) -> dict:  # type: ignore
         """ Prepare request """
 
-        request = super().prepare(proxy=proxy, cookies=cookies)
+        # default values for additional params
+        # see https://2captcha.com/2captcha-api#solving_tiktok for details
+        add_params_mapping = {
+            'https://www.tiktok.com/login/phone-or-email/email': {
+                'aid': 1459,
+                'host': 'https://www-useast1a.tiktok.com'
+            },
+            'https://ads.tiktok.com/i18n/signup': {
+                'aid': 1583,
+                'host': 'https://verify-sg.byteoversea.com'
+            },
+            'default': {'aid': None, 'host': None}
+        }
+
+        add_params = add_params_mapping[next(
+            (url for url in add_params_mapping if captcha.page_url.startswith(url)),
+            'default'
+        )]
+
+        request = super().prepare(
+            captcha=captcha,
+            proxy=proxy,
+            user_agent=user_agent,
+            cookies=cookies
+        )
+
         request['data'].update(
             dict(
                 method="tiktok",
-                pageurl=captcha.page_url
+                pageurl=captcha.page_url,
+                aid=add_params['aid'] if captcha.aid is None else captcha.aid,
+                host=add_params['host'] if captcha.host is None else captcha.host
             )
         )
 
@@ -581,3 +691,32 @@ class TikTokCaptchaTaskRequest(TaskRequest):
 
 class TikTokCaptchaSolutionRequest(SolutionRequest):
     """ TikTokCaptcha solution request """
+
+
+class GeeTestV4TaskRequest(TaskRequest):
+    """ GeeTest v4 task request """
+
+    # pylint: disable=arguments-differ,signature-differs
+    def prepare(self, captcha, proxy, user_agent, cookies) -> dict:  # type: ignore
+        """ Prepare request """
+
+        request = super().prepare(
+            captcha=captcha,
+            proxy=proxy,
+            user_agent=user_agent,
+            cookies=cookies
+        )
+
+        request['data'].update(
+            dict(
+                method="geetest_v4 ",
+                captcha_id=captcha.captcha_id,
+                pageurl=captcha.page_url
+            )
+        )
+
+        return request
+
+
+class GeeTestV4SolutionRequest(SolutionRequest):
+    """ GeeTest v4 solution request """
